//*********************************************************************************
//		CLASS				Pluto95Applet - default package
//*********************************************************************************

import Pluto95;
import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Container;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.TextArea;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.awt.TextField;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/******************************************************************************
Applet embedding access to pluto coordinates, via Pluto95.java.
<BR/>See <CODE>Pluto95.java</CODE> source code for more information.
<BR/>Licenced under the terms of the General Public Licence, available at 
<A HREF="xxx">xxx</A>
<BR/>Author : Thierry Graff, tigraff@chez.com
<BR/>History :
<LI>nov 19 2000 : creation and tests.</LI>
@todo address of GPL
*********************************************************************************/
public class Pluto95Applet extends Applet {

  CalcListener theCalcListener;
  Button btnCalc;
  TextField txtDate;
  TextField txtHour;
  TextArea resArea;

	double[][] planetPos = new double[12][4];
	double[][] planetVel = new double[12][4];

  public void init() {

    // *** GUI construction

    // text fields
    Label lblDate = new Label("Date : ");
    Label lblDateFormat = new Label("(format MM/DD/YYYY)");
    Label lblHour = new Label("Hour : ");
    Label lblHourFormat = new Label("(format hh/mm/ss)");
    txtDate = new TextField(10);
    txtHour = new TextField(8);

    Panel pd1 = new Panel();
    pd1.add(lblDate);
    pd1.add(txtDate);
    Panel pd2 = new Panel(); // just to center
    pd2.add(lblDateFormat);
    Panel ph1 = new Panel();
    ph1.add(lblHour);
    ph1.add(txtHour);
    Panel ph2 = new Panel(); // just to center
    ph2.add(lblHourFormat);
    Panel p1 =  new Panel();
    p1.setLayout(new GridLayout(0,2));
    p1.add(pd1);
    p1.add(ph1);
    p1.add(pd2);
    p1.add(ph2);
    
    // button
		btnCalc = new Button("Calculate");
		//btnCalc.setToolTipText("Calculate positions for this date");
		//btnCalc.setMnemonic('c');
		theCalcListener = new CalcListener();
		btnCalc.addActionListener(theCalcListener);
    
    Panel pc1 = new Panel();
    pc1.add(btnCalc);
    
    //resultPane
		resArea = new TextArea("", 15, 30, TextArea.SCROLLBARS_VERTICAL_ONLY);
    resArea.setBackground(Color.white);
		resArea.setEditable(false);
    
    //layout
    Panel p2 =  new Panel();
    p2.setLayout(new GridLayout(0,1));
    p2.add(p1);
    p2.add(pc1);
    
    this.add(p2, BorderLayout.NORTH);
    this.add(resArea, BorderLayout.CENTER);

    //this.validate();        
  }// init

	//**************************************
	class CalcListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
    
      String strDate = txtDate.getText();      
      String strHour = txtHour.getText();      
      int retCode;
      
      int[] date = {0, 0, 0, 0, 0, 0};
      retCode = parseDate(strDate, strHour, "en", date);
      if (retCode != 0){
        resArea.append("Invalid date or hour entered\n");
        return;
      }

      double jd = dateToJD(date[0], date[1], date[2], date[3], date[4], date[5]);

      double res[] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      retCode = Pluto95.calcPluto(jd, res, true);
      if (retCode != 0){
        resArea.append("Date out of range. \n");
        resArea.append("Must be between :\nJan 01 1700 0h and Jan 24 2100 0h. \n");
        return;
      }
      
      if (strDate.equals("")) strDate = "00h00m00s";
      resArea.append("Date : " + addZero(date[1]) + "/"
                               + addZero(date[2]) + "/"
                               + String.valueOf(date[0]) + " - "
                               + addZero(date[3]) + "h"
                               + addZero(date[4]) + "m" 
                               + addZero(date[5]) + "s" + "\n");
      resArea.append("Julian Date : " + String.valueOf(jd) + "\n");

    String coords[] = {"X", "Y", "Z", "X'", "Y'", "Z'"};
    String units[] = {"ua", "ua", "ua", "ua/d", "ua/d", "ua/d"};
    for (int i=0;i<6;i++)
      resArea.append(coords[i] + " = " + String.valueOf(res[i]) + " " + units[i] + "\n");

      resArea.append("\n============================\n\n");
		} //end actionPerformed
	} //end class Quit Listener

	//**************************** dateToJD ***************************************
	/******************************************************************************
		Calculates julian day of a date, specifying date and hour.
    <BR/>Used in different autonom programs ; REFERENCE IMPLEMENTATION.
    @todo test with negative years
	******************************************************************************/
	public static double dateToJD(int year, int month, int day, int hour, int minut, int sec){
	// @todo when year <0.
    double jd; 
		double y1, m1;

  	double decDay = day + hour/24.0 + minut/1440.0 + sec/86400.0;

		if(month>2){
			y1 = year;
			m1 = month;
		}
		else{
			y1 = year-1;
			m1 = month+12;
		}
    if (year > 0)
  		jd = Math.floor(365.25*y1) + Math.floor(30.6001*(m1+1)) + decDay + 1720994.5;
    else
  		jd = Math.floor(365.25*y1 - 0.75) + Math.floor(30.6001*(m1+1)) + decDay + 1720994.5;

		if((year+month/100.0 + decDay/10000.0) >= 1582.1015){
			double a = Math.floor(y1/100.0);
			double b = 2-a+Math.floor(a/4.0);
			jd = jd + b;
		}
		return jd;
  }// end dateToJD
    
	//**************************** parseDate ***************************************
	/******************************************************************************
		Parses date and hour strings.
    <BR/>If strHour is empty, h, minut, s will be set to 0.
    @param strDate date ; fields must be separated by any non-numeric separator. 
    Examples of valid input : "18/07/1968" (french date) ; "12:13:2004" (english date).
    @param strHour hour ; fields must be separated by any non-numeric separator.
    Examples of valid input : "12h40m15s", "12:40", "12", "12.40.00".
    @param lang language code, as specified by ISO machin (supported languages : "en" and "fr").
    @param results array onf integers in which are stored (in this order) years, months, days,
           hours, minuts, seconds.
    @return code indicating if the strings were valid :
    <BR/>0 : OK, parsing successful.
    <BR/>1 : wrong or unsupported language code.
    <BR/>2 : wrong date format.
    <BR/>3 : wrong hour format.
    <BR/>4 : bad dimension of parameter "results".
	******************************************************************************/
	public static int parseDate(String strDate, String strHour, String lang, int[] results){
 
    if (results.length != 6) return 4;
    int len,i;
    String str1, str2, str3;
    
    // 1 - Date parsing
    len = strDate.length();
    str1=str2=str3="";
    i=0;
    // str1
    while(i<len && Character.isDigit(strDate.charAt(i))){
      str1 += String.valueOf(strDate.charAt(i));
      i++;
    }
    // separator
    while(i<len && !Character.isDigit(strDate.charAt(i)))
      i++;
    // str2
    while(i<len && Character.isDigit(strDate.charAt(i))){
      str2 += String.valueOf(strDate.charAt(i));
      i++;
    }
    // separator
    while(i<len && !Character.isDigit(strDate.charAt(i)))
      i++;
    // str3
    while(i<len && Character.isDigit(strDate.charAt(i))){
      str3 += String.valueOf(strDate.charAt(i));
      i++;
    }
    //return code
    if (i != len || str1==""  || str2==""  || str3=="") return 2;
    if (lang.equals("fr")){
      results[1] = (Integer.valueOf(str2)).intValue(); // months
      results[2] = (Integer.valueOf(str1)).intValue(); //days
    }
    else if (lang.equals("en")){
      results[1] = (Integer.valueOf(str1)).intValue(); //days
      results[2] = (Integer.valueOf(str2)).intValue(); // months
    }
    else
      return 1;
    results[0] = (Integer.valueOf(str3)).intValue(); // years

    // 2 - Hour parsing
    len = strHour.length();
    if (len == 0){ // hour not specified
      results[3] = 0; results[4] = 0; results[5] = 0; // hours, minuts, seconds
      return 0;
    }
    str1=str2=str3="";
    i=0;
    // str1
    while(i<len && Character.isDigit(strHour.charAt(i))){
      str1 += String.valueOf(strHour.charAt(i));
      i++;
    }
    // separator
    while(i<len && !Character.isDigit(strHour.charAt(i)))
      i++;
    // str2
    while(i<len && Character.isDigit(strHour.charAt(i))){
      str2 += String.valueOf(strHour.charAt(i));
      i++;
    }
    // separator
    while(i<len && !Character.isDigit(strHour.charAt(i)))
      i++;
    // str3
    while(i<len && Character.isDigit(strHour.charAt(i))){
      str3 += String.valueOf(strHour.charAt(i));
      i++;
    }

    if (str1.equals("")){
      results[3] = 0; results[4] = 0; results[5] = 0; // hours, minuts, seconds
      return 0;
    }
    results[3] = (Integer.valueOf(str1)).intValue(); // hours
    
    if (str2.equals("")){
      results[4] = 0; results[5] = 0; // minuts, seconds
      return 0;
    }
    results[4] = (Integer.valueOf(str2)).intValue(); // minuts
    
    if (str3.equals("")){
      results[5] = 0; // seconds
      return 0;
     }
    results[5] = (Integer.valueOf(str3)).intValue(); // seconds
    
    return 0;

  }// end parseDate 

	//**************************** addZero ***************************************
	/******************************************************************************
    Formats a integer to a 2 character long String. Useful for date formatting.
  ******************************************************************************/
	public static String addZero(int nb){
    if (nb<0 || nb>99) return "xx";
    if (nb<10) return "0" + String.valueOf(nb);
    else return String.valueOf(nb);
  }// end addZero

}// end Pluto95Applet
